#ifndef ETN_FPGA_MGR_MAIN_ALTERA__H
#define ETN_FPGA_MGR_MAIN_ALTERA__H

#include <linux/of.h>
#include <linux/fpga.h>

/* Controls whether to use the Configuration with DCLK steps */
#ifndef _ALT_FPGA_USE_DCLK
#define _ALT_FPGA_USE_DCLK 0
#endif

/* Register offsets */
#define ALT_FPGAMGR_STAT_OFST				0x0
#define ALT_FPGAMGR_CTL_OFST				0x4
#define ALT_FPGAMGR_DCLKCNT_OFST			0x8
#define ALT_FPGAMGR_DCLKSTAT_OFST			0xc
#define ALT_FPGAMGR_MON_GPIO_INTEN_OFST			0x830
#define ALT_FPGAMGR_MON_GPIO_INTMSK_OFST		0x834
#define ALT_FPGAMGR_MON_GPIO_INTTYPE_LEVEL_OFST		0x838
#define ALT_FPGAMGR_MON_GPIO_INT_POL_OFST		0x83c
#define ALT_FPGAMGR_MON_GPIO_INTSTAT_OFST		0x840
#define ALT_FPGAMGR_MON_GPIO_RAW_INTSTAT_OFST		0x844
#define ALT_FPGAMGR_MON_GPIO_PORTA_EOI_OFST		0x84c
#define ALT_FPGAMGR_MON_GPIO_EXT_PORTA_OFST		0x850

/* Register bit defines */
/* ALT_FPGAMGR_STAT register */
#define ALT_FPGAMGR_STAT_POWER_UP			0x0
#define ALT_FPGAMGR_STAT_RESET				0x1
#define ALT_FPGAMGR_STAT_CFG				0x2
#define ALT_FPGAMGR_STAT_INIT				0x3
#define ALT_FPGAMGR_STAT_USER_MODE			0x4
#define ALT_FPGAMGR_STAT_UNKNOWN			0x5
#define ALT_FPGAMGR_STAT_STATE_MASK			0x7
/* This is a flag value that doesn't really happen in this register field */
#define ALT_FPGAMGR_STAT_POWER_OFF			0xf

#define MSEL_PP16_FAST_NOAES_NODC			0x0
#define MSEL_PP16_FAST_AES_NODC				0x1
#define MSEL_PP16_FAST_AESOPT_DC			0x2
#define MSEL_PP16_SLOW_NOAES_NODC			0x4
#define MSEL_PP16_SLOW_AES_NODC				0x5
#define MSEL_PP16_SLOW_AESOPT_DC			0x6
#define MSEL_PP32_FAST_NOAES_NODC			0x8
#define MSEL_PP32_FAST_AES_NODC				0x9
#define MSEL_PP32_FAST_AESOPT_DC			0xa
#define MSEL_PP32_SLOW_NOAES_NODC			0xc
#define MSEL_PP32_SLOW_AES_NODC				0xd
#define MSEL_PP32_SLOW_AESOPT_DC			0xe
#define ALT_FPGAMGR_STAT_MSEL_MASK			0x000000f8
#define ALT_FPGAMGR_STAT_MSEL_SHIFT			3

/* ALT_FPGAMGR_CTL register */
#define ALT_FPGAMGR_CTL_EN				0x00000001
#define ALT_FPGAMGR_CTL_NCE				0x00000002
#define ALT_FPGAMGR_CTL_NCFGPULL			0x00000004

#define CDRATIO_X1					0x00000000
#define CDRATIO_X2					0x00000040
#define CDRATIO_X4					0x00000080
#define CDRATIO_X8					0x000000c0
#define ALT_FPGAMGR_CTL_CDRATIO_MASK			0x000000c0

#define ALT_FPGAMGR_CTL_AXICFGEN			0x00000100

#define CFGWDTH_16					0x00000000
#define CFGWDTH_32					0x00000200
#define ALT_FPGAMGR_CTL_CFGWDTH_MASK			0x00000200

/* ALT_FPGAMGR_DCLKSTAT register */
#define ALT_FPGAMGR_DCLKSTAT_DCNTDONE_E_DONE		0x1

/* ALT_FPGAMGR_MON_GPIO_* registers share the same bit positions */
#define ALT_FPGAMGR_MON_NSTATUS				0x0001
#define ALT_FPGAMGR_MON_CONF_DONE			0x0002
#define ALT_FPGAMGR_MON_INIT_DONE			0x0004
#define ALT_FPGAMGR_MON_CRC_ERROR			0x0008
#define ALT_FPGAMGR_MON_CVP_CONF_DONE			0x0010
#define ALT_FPGAMGR_MON_PR_READY			0x0020
#define ALT_FPGAMGR_MON_PR_ERROR			0x0040
#define ALT_FPGAMGR_MON_PR_DONE				0x0080
#define ALT_FPGAMGR_MON_NCONFIG_PIN			0x0100
#define ALT_FPGAMGR_MON_NSTATUS_PIN			0x0200
#define ALT_FPGAMGR_MON_CONF_DONE_PIN			0x0400
#define ALT_FPGAMGR_MON_FPGA_POWER_ON			0x0800
#define ALT_FPGAMGR_MON_STATUS_MASK			0x0fff

/* Reset management registers (for now only brgmodbrst) */
#define   RSTMGR_ADDR         0xFFD05000
#define   RSTMGR_LEN          0x24
#define   BRGMODRST_REG_OFF   0x1C

/* BRGMODRST fields */
#define   BRGMODRST_MASK       0x7 // 3 bits
#define   BRGMODRST_F2H_BIT    1 << 2
#define   BRGMODRST_LWH2F_BIT  1 << 1
#define   BRGMODRST_H2F_BIT    1 << 0

/* L3 GPV registers (for now only remap) */
#define   L3REGS_ADDR        0xFF800000
#define   L3REGS_LEN         0x100000
#define   L3REGS_REMAP_OFF   0x0  /* Remap register */
#define   L3REGS_SECUR_OFF   0x20 /* Security settings register (lwhps2fpgaregs) */

/* remap fields */
#define   L3REGS_REMAP_LWH2F_BIT  1 << 4
#define   L3REGS_REMAP_H2F_BIT    1 << 3

/* FPGAPORTRST register */
#define FPGAPORTRST_ADDR 0xFFC25080
#define FPGAPORTRST_LEN 4 /* Single register */
#define FPGAPORTRST_DISABLE_ALL 0x0
#define FPGAPORTRST_ENABLE_ALL 0x3fff

struct cfgmgr_mode {
	/* Values to set in the CTRL register */
	u32 ctrl;

	/* flag that this table entry is a valid mode */
	bool valid;
};

static const char *const altera_fpga_state_str[] = {
	[ALT_FPGAMGR_STAT_POWER_UP] = "power up phase",
	[ALT_FPGAMGR_STAT_RESET] = "reset phase",
	[ALT_FPGAMGR_STAT_CFG] = "configuration phase",
	[ALT_FPGAMGR_STAT_INIT] = "initialization phase",
	[ALT_FPGAMGR_STAT_USER_MODE] = "user mode",
	[ALT_FPGAMGR_STAT_UNKNOWN] = "undetermined",
	[ALT_FPGAMGR_STAT_POWER_OFF] = "powered off",
};

int alt_fpga_state_get(struct fpga_manager *mgr);
int alt_fpga_load_fw(char *fw_data, size_t fw_size, struct fpga_manager *mgr);

extern struct fpga_manager_ops altera_fpga_mgr_ops;

#endif // ETN_FPGA_MGR_MAIN_ALTERA__H
