#ifndef ETN_IO_MAIN__H
#define ETN_IO_MAIN__H

#include <linux/miscdevice.h>
#include <linux/mutex.h>
#include <linux/spinlock.h>

#define DRV_NAME "etn_io"

#ifndef DRV_VERSION
#define DRV_VERSION "1.0.1"
#endif

#define DEV_NAME "etn"

/* Use mutex, because it's safe to 
 * sleep inside mutex critical section.*/
#define DECLARE_LOCK(lock) struct mutex lock
#define LOCK_INIT(lock) mutex_init(lock)
#define LOCK(lock)      mutex_lock(lock)
#define UNLOCK(lock)    mutex_unlock(lock)
#define TRYLOCK(lock)	mutex_trylock(lock)

/* Maybe we should try spin_lock_irqsave? */
/*
 *#define DECLARE_LOCK(lock) spinlock_t lock
 *#define LOCK_INIT(lock) spin_lock_init(lock)
 *#define LOCK(lock)      spin_lock(lock)
 *#define UNLOCK(lock)    spin_unlock(lock)
 *#define TRYLOCK(lock)	spin_trylock(lock)
 */

struct fpga_dev {
	struct platform_device *pdev;
	struct fpga_io         *io;
	struct miscdevice      miscdev;

	void __iomem *H2F;   /* HPS to FPGA interface */
	void __iomem *LWH2F; /* Lightweight HPS to FPGA interface */
};

int etn_register(struct fpga_dev *fpga);
void etn_unregister(struct fpga_dev *fpga);


#endif // ETN_IO_MAIN__H
