#ifndef ETN_NET_ALT_MSGDMA__H
#define ETN_NET_ALT_MSGDMA__H

/*
 * These come from csr_regs.h in the design example
 */
#define ALT_MSGDMA_CSR_SIZE 32

struct alt_msgdma_csr {
	u32 status;
	u32 control;
	u16 read_fill_level;
	u16 write_fill_level;
	u16 response_fill_level;
	u16 reserved1;
	u16 read_seq;
	u16 write_seq;
	u32 reserved2;
	u32 reserved3;
	u32 reserved4;
} __attribute__ ((packed, aligned(ALT_MSGDMA_CSR_SIZE)));

#define CSR_STATUS_REG                                  (0x0)
#define CSR_CONTROL_REG                                 (0x4)
#define CSR_DESCRIPTOR_FILL_LEVEL_REG                   (0x8)
#define CSR_RESPONSE_FILL_LEVEL_REG                     (0xC)
/* this register only exists when the enhanced features are enabled */
#define CSR_SEQUENCE_NUMBER_REG                         (0x10)

/* masks for the status register bits */
#define CSR_BUSY_MASK                                   (1)
#define CSR_BUSY_OFFSET                                 (0)
#define CSR_DESCRIPTOR_BUFFER_EMPTY_MASK                (1<<1)
#define CSR_DESCRIPTOR_BUFFER_EMPTY_OFFSET              (1)
#define CSR_DESCRIPTOR_BUFFER_FULL_MASK                 (1<<2)
#define CSR_DESCRIPTOR_BUFFER_FULL_OFFSET               (2)
#define CSR_RESPONSE_BUFFER_EMPTY_MASK                  (1<<3)
#define CSR_RESPONSE_BUFFER_EMPTY_OFFSET                (3)
#define CSR_RESPONSE_BUFFER_FULL_MASK                   (1<<4)
#define CSR_RESPONSE_BUFFER_FULL_OFFSET                 (4)
#define CSR_STOP_STATE_MASK                             (1<<5)
#define CSR_STOP_STATE_OFFSET                           (5)
#define CSR_RESET_STATE_MASK                            (1<<6)
#define CSR_RESET_STATE_OFFSET                          (6)
#define CSR_STOPPED_ON_ERROR_MASK                       (1<<7)
#define CSR_STOPPED_ON_ERROR_OFFSET                     (7)
#define CSR_STOPPED_ON_EARLY_TERMINATION_MASK           (1<<8)
#define CSR_STOPPED_ON_EARLY_TERMINATION_OFFSET         (8)
#define CSR_IRQ_SET_MASK                                (1<<9)
#define CSR_IRQ_SET_OFFSET                              (9)

/* masks for the control register bits */
#define CSR_STOP_MASK                                   (1)
#define CSR_STOP_OFFSET                                 (0)
#define CSR_RESET_MASK                                  (1<<1)
#define CSR_RESET_OFFSET                                (1)
#define CSR_STOP_ON_ERROR_MASK                          (1<<2)
#define CSR_STOP_ON_ERROR_OFFSET                        (2)
#define CSR_STOP_ON_EARLY_TERMINATION_MASK              (1<<3)
#define CSR_STOP_ON_EARLY_TERMINATION_OFFSET            (3)
#define CSR_GLOBAL_INTERRUPT_MASK                       (1<<4)
#define CSR_GLOBAL_INTERRUPT_OFFSET                     (4)
#define CSR_STOP_DESCRIPTORS_MASK                       (1<<5)
#define CSR_STOP_DESCRIPTORS_OFFSET                     (5)

/* masks for the FIFO fill levels and sequence number */
#define CSR_READ_FILL_LEVEL_MASK                        (0xFFFF)
#define CSR_READ_FILL_LEVEL_OFFSET                      (0)
#define CSR_WRITE_FILL_LEVEL_MASK                       (0xFFFF0000)
#define CSR_WRITE_FILL_LEVEL_OFFSET                     (16)
#define CSR_RESPONSE_FILL_LEVEL_MASK                    (0xFFFF)
#define CSR_RESPONSE_FILL_LEVEL_OFFSET                  (0)
#define CSR_READ_SEQUENCE_NUMBER_MASK                   (0xFFFF)
#define CSR_READ_SEQUENCE_NUMBER_OFFSET                 (0)
#define CSR_WRITE_SEQUENCE_NUMBER_MASK                  (0xFFFF0000)
#define CSR_WRITE_SEQUENCE_NUMBER_OFFSET                (16)

/*
 * These come from descriptor_regs.h in the design example
 */
#define DESCRIPTOR_READ_ADDRESS_REG                     (0x0)
#define DESCRIPTOR_WRITE_ADDRESS_REG                    (0x4)
#define DESCRIPTOR_LENGTH_REG                           (0x8)
#define DESCRIPTOR_CONTROL_STANDARD_REG                 (0xC)
#define DESCRIPTOR_SEQUENCE_NUMBER_REG                  (0xC)
#define DESCRIPTOR_READ_BURST_REG                       (0xE)
#define DESCRIPTOR_WRITE_BURST_REG                      (0xF)
#define DESCRIPTOR_READ_STRIDE_REG                      (0x10)
#define DESCRIPTOR_WRITE_STRIDE_REG                     (0x12)
#define DESCRIPTOR_READ_ADDRESS_HIGH_REG                (0x14)
#define DESCRIPTOR_WRITE_ADDRESS_HIGH_REG               (0x18)
#define DESCRIPTOR_CONTROL_ENHANCED_REG                 (0x1C)

/* masks and offsets for the sequence number and programmable burst counts */
#define DESCRIPTOR_SEQUENCE_NUMBER_MASK                 (0xFFFF)
#define DESCRIPTOR_SEQUENCE_NUMBER_OFFSET               (0)
#define DESCRIPTOR_READ_BURST_COUNT_MASK                (0x00FF0000)
#define DESCRIPTOR_READ_BURST_COUNT_OFFSET              (16)
#define DESCRIPTOR_WRITE_BURST_COUNT_MASK               (0xFF000000)
#define DESCRIPTOR_WRITE_BURST_COUNT_OFFSET             (24)

/* masks and offsets for the read and write strides */
#define DESCRIPTOR_READ_STRIDE_MASK                     (0xFFFF)
#define DESCRIPTOR_READ_STRIDE_OFFSET                   (0)
#define DESCRIPTOR_WRITE_STRIDE_MASK                    (0xFFFF0000)
#define DESCRIPTOR_WRITE_STRIDE_OFFSET                  (16)

/* masks and offsets for the bits in the descriptor control field */
#define DESCRIPTOR_CONTROL_TRANSMIT_CHANNEL_MASK        (0xFF)
#define DESCRIPTOR_CONTROL_TRANSMIT_CHANNEL_OFFSET      (0)
#define DESCRIPTOR_CONTROL_GENERATE_SOP_MASK            (1<<8)
#define DESCRIPTOR_CONTROL_GENERATE_SOP_OFFSET          (8)
#define DESCRIPTOR_CONTROL_GENERATE_EOP_MASK            (1<<9)
#define DESCRIPTOR_CONTROL_GENERATE_EOP_OFFSET          (9)
#define DESCRIPTOR_CONTROL_PARK_READS_MASK              (1<<10)
#define DESCRIPTOR_CONTROL_PARK_READS_OFFSET            (10)
#define DESCRIPTOR_CONTROL_PARK_WRITES_MASK             (1<<11)
#define DESCRIPTOR_CONTROL_PARK_WRITES_OFFSET           (11)
#define DESCRIPTOR_CONTROL_END_ON_EOP_MASK              (1<<12)
#define DESCRIPTOR_CONTROL_END_ON_EOP_OFFSET            (12)
#define DESCRIPTOR_CONTROL_TRANSFER_COMPLETE_IRQ_MASK   (1<<14)
#define DESCRIPTOR_CONTROL_TRANSFER_COMPLETE_IRQ_OFFSET (14)
#define DESCRIPTOR_CONTROL_EARLY_TERMINATION_IRQ_MASK   (1<<15)
#define DESCRIPTOR_CONTROL_EARLY_TERMINATION_IRQ_OFFSET (15)
/* the read master will use this as the transmit error, the dispatcher
   will use this to generate an interrupt if any of the error bits are
   asserted by the write master */
#define DESCRIPTOR_CONTROL_ERROR_IRQ_MASK               (0xFF<<16)
#define DESCRIPTOR_CONTROL_ERROR_IRQ_OFFSET             (16)
#define DESCRIPTOR_CONTROL_EARLY_DONE_ENABLE_MASK       (1<<24)
#define DESCRIPTOR_CONTROL_EARLY_DONE_ENABLE_OFFSET     (24)
/* at a minimum you always have to write '1' to this bit as it commits
   the descriptor to the dispatcher */
#define DESCRIPTOR_CONTROL_GO_MASK                      (1<<31)
#define DESCRIPTOR_CONTROL_GO_OFFSET                    (31)

/*
 * These come from response_regs.h in the design example
 */
#define RESPONSE_ACTUAL_BYTES_TRANSFERRED_REG           (0x0)
#define RESPONSE_ERRORS_REG                             (0x4)

/* bits making up the "errors" register */
#define RESPONSE_ERROR_MASK                             (0xFF)
#define RESPONSE_ERROR_OFFSET                           (0)
#define RESPONSE_EARLY_TERMINATION_MASK                 (1<<8)
#define RESPONSE_EARLY_TERMINATION_OFFSET               (8)

/*
 * These come from sgdma_dispatcher.h in the design example
 */
#define ALT_MSGDMA_STANDARD_DESCRIPTOR_SIZE 16
#define ALT_MSGDMA_EXTENDED_DESCRIPTOR_SIZE 32
#define ALT_MSGDMA_RESPONSE_SIZE 8

struct alt_msgdma_standard_descriptor {
	u32 read_address;
	u32 write_address;
	u32 transfer_length;
	u32 control;
} __attribute__ ((packed, aligned(ALT_MSGDMA_STANDARD_DESCRIPTOR_SIZE)));

struct alt_msgdma_extended_descriptor {
	u32 read_address;
	u32 write_address;
	u32 transfer_length;
	u16 sequence_number;
	u8  read_burst_count;
	u8  write_burst_count;
	u16 read_stride;
	u16 write_stride;
	u32 read_address_high;
	u32 write_address_high;
	u32 control;
} __attribute__ ((packed, aligned(ALT_MSGDMA_EXTENDED_DESCRIPTOR_SIZE)));

/* this struct should only be used if response information is enabled */
struct alt_msgdma_response {
	u32 actual_bytes_transferred;
	union {
		struct {
			u8 error;
			u8 early_termination;
		};
		u32 errors;
	};
} __attribute__ ((packed, aligned(ALT_MSGDMA_RESPONSE_SIZE)));

#endif /* ETN_NET_ALT_MSGDMA__H */
